/*
 * $Id$
 */


/*------------------------------------------------------------------------

    Datei COMMAND.H zum Projekt RTF2TEX.C

    Struktur mit RTF-Kommandos und Status
    "Ubersetzungstabellen f"ur Hexadezimalzahlen und Zeichen
        
    letzte "Anderung: am 4.9.1991 um 17:20

------------------------------------------------------------------------*/

#include "rtf2tex.h"

/*------------------------------------------------------------------------
    Tabelle der RTF-Kommandos
    Spalte 1: RTF-Kommando
           2: Befehlsstatus (IGNORE, COMMENT, EXECUTE, WARN)
           3: Pointer auf Codegenerierungsroutine   
------------------------------------------------------------------------*/

struct command Cmdtab[]=
{
    "deff",         IGNORE, NULL,   /* nicht dokumentierter Befehl !!!  */      
    "chpgn",        IGNORE, NULL,
    "chftn",        IGNORE, NULL,
    "chdate",       IGNORE, NULL,
    "chtime",       IGNORE, NULL,
    "|",            IGNORE, NULL,
    "~",            IGNORE, NULL,
    "-",            IGNORE, NULL,
    "_",            IGNORE, NULL,
    "page",         EXECUTE,gen_page,
    "line",         EXECUTE,gen_line,
    "par",          EXECUTE,gen_par,
    "sect",         IGNORE, NULL,
    "tab",          EXECUTE,gen_tab,
    "rtf",          IGNORE, NULL,
    "pc",           EXECUTE,gen_pc,
    "ansi",         EXECUTE,gen_ansi,
    "mac",          EXECUTE,gen_mac,
    "colortbl",     EXECUTE,gen_colortbl,
    "red",          IGNORE, NULL,
    "green",        IGNORE, NULL,
    "blue",         IGNORE, NULL,
    "fonttbl",      EXECUTE,gen_fonttbl,
    "stylesheet",   EXECUTE,gen_stylesheet,
    "pict",         IGNORE, NULL,
    "footnote",     EXECUTE,gen_footnote,
    "header",       IGNORE, NULL,
    "headerl",      IGNORE, NULL,
    "headerr",      IGNORE, NULL,
    "headerf",      IGNORE, NULL,
    "footer",       IGNORE, NULL,
    "footerl",      IGNORE, NULL,
    "footerr",      IGNORE, NULL,
    "footerf",      IGNORE, NULL,
    "ftnsep",       IGNORE, NULL,
    "ftnsepc",      IGNORE, NULL,
    "ftncn",        IGNORE, NULL,
    "info",         EXECUTE,gen_info,
    "verno",        IGNORE, NULL,
    "yr",           IGNORE, NULL,
    "mo",           IGNORE, NULL,
    "dy",           IGNORE, NULL,
    "hr",           IGNORE, NULL,
    "min",          IGNORE, NULL,
    "sec",          IGNORE, NULL,
    "creatim",      IGNORE, NULL,
    "revtim",       IGNORE, NULL,
    "printtim",     IGNORE, NULL,
    "butim",        IGNORE, NULL,
    "edmins",       IGNORE, NULL,
    "nofpages",     IGNORE, NULL,
    "nofwords",     IGNORE, NULL,
    "nofchars",     IGNORE, NULL,
    "id",           IGNORE, NULL,
    "comment",      IGNORE, NULL,
    "paperw",       EXECUTE,gen_commentdocsize,
    "paperh",       EXECUTE,gen_commentdocsize,
    "margl",        EXECUTE,gen_commentdocsize,
    "margr",        EXECUTE,gen_commentdocsize,
    "margt",        EXECUTE,gen_commentdocsize,
    "margb",        EXECUTE,gen_commentdocsize,
    "facingp",      IGNORE, NULL,
    "gutter",       IGNORE, NULL,
    "ogutter",      IGNORE, NULL,
    "deftab",       IGNORE, NULL,
    "widowctrl",    IGNORE, NULL,   /* Fehler in Dokumentation !!!      */
    "endnotes",     IGNORE, NULL,
    "ftnbj",        IGNORE, NULL,
    "ftntj",        IGNORE, NULL,
    "ftnstart",     IGNORE, NULL,
    "ftnrestart",   IGNORE, NULL,   /* nicht dokumentierter Befehl !!!  */
    "pgnstart",     IGNORE, NULL,
    "linestart",    IGNORE, NULL,
    "landscape",    IGNORE, NULL,
    "sectd",        IGNORE, NULL,
    "sbknone",      IGNORE, NULL,
    "sbkcol",       IGNORE, NULL,
    "sbkpage",      IGNORE, NULL,
    "sbkeven",      IGNORE, NULL,
    "sbkodd",       IGNORE, NULL,
    "pgnrestart",   IGNORE, NULL,
    "pgndec",       IGNORE, NULL,
    "pgnucrm",      IGNORE, NULL,
    "pgnlcrm",      IGNORE, NULL,
    "pgnucltr",     IGNORE, NULL,
    "pgnlcltr",     IGNORE, NULL,
    "pgnx",         IGNORE, NULL,
    "pgny",         IGNORE, NULL,
    "linemod",      IGNORE, NULL,
    "linex",        IGNORE, NULL,
    "linerestart",  IGNORE, NULL,
    "linepage",     IGNORE, NULL,
    "linecont",     IGNORE, NULL,
    "headery",      IGNORE, NULL,
    "footery",      IGNORE, NULL,
    "vertalt",      IGNORE, NULL,
    "vertalc",      IGNORE, NULL,
    "vertalj",      IGNORE, NULL,
    "vertalb",      IGNORE, NULL,
    "cols",         IGNORE, NULL,
    "colsx",        IGNORE, NULL,
    "endnhere",     IGNORE, NULL,
    "titlepg",      IGNORE, NULL,
    "pard",         EXECUTE,gen_pard,
    "s",            IGNORE, NULL,
    "ql",           EXECUTE,gen_ql,
    "qr",           EXECUTE,gen_qr,
    "qc",           EXECUTE,gen_qc,
    "qj",           EXECUTE,gen_qj,
    "fi",           EXECUTE,gen_fi,
    "li",           EXECUTE,gen_li,
    "ri",           EXECUTE,gen_ri,
    "sb",           IGNORE, NULL,
    "sa",           IGNORE, NULL,
    "sl",           IGNORE, NULL,
    "keep",         IGNORE, NULL,
    "keepn",        IGNORE, NULL,
    "sbys",         IGNORE, NULL,
    "pagebb",       IGNORE, NULL,
    "noline",       IGNORE, NULL,
    "brdrt",        IGNORE, NULL,
    "brdrb",        IGNORE, NULL,
    "brdrl",        IGNORE, NULL,
    "brdrr",        IGNORE, NULL,
    "box",          IGNORE, NULL,
    "brdrs",        IGNORE, NULL,
    "brdrth",       IGNORE, NULL,
    "brdsh",        IGNORE, NULL,
    "brdrdb",       IGNORE, NULL,
    "tqr",          IGNORE, NULL,
    "tqc",          IGNORE, NULL,
    "tqdec",        IGNORE, NULL,
    "tldot",        IGNORE, NULL,
    "tlhyphen",     IGNORE, NULL,
    "tlul",         IGNORE, NULL,
    "tlth",         IGNORE, NULL,
    "tx",           IGNORE, NULL,
    "tb",           IGNORE, NULL,
    "plain",        EXECUTE,gen_plain,
    "b",            EXECUTE,gen_b,
    "i",            EXECUTE,gen_i,
    "strike",       IGNORE, NULL,
    "outl",         IGNORE, NULL,
    "shad",         IGNORE, NULL,
    "scaps",        EXECUTE,gen_scaps,
    "caps",         IGNORE, NULL,
    "v",            EXECUTE,gen_v,
    "f",            IGNORE, NULL,
    "fs",           EXECUTE,gen_fs,
    "expnd",        IGNORE, NULL,
    "ul",           EXECUTE,gen_ul,
    "ulw",          IGNORE, NULL,
    "uld",          IGNORE, NULL,
    "uldb",         EXECUTE,gen_uldb,
    "up",           EXECUTE,gen_up,
    "dn",           EXECUTE,gen_dn,
    "cf",           IGNORE, NULL,
    "cb",           IGNORE, NULL,
    "sbasedon",     IGNORE, NULL,
    "snext",        IGNORE, NULL,
    "pich",         IGNORE, NULL,
    "picw",         IGNORE, NULL,
    "picscaled",    IGNORE, NULL,
    "wmetafile",    IGNORE, NULL,
    "macpict",      IGNORE, NULL,
    "bin",          IGNORE, NULL,
    "title",        EXECUTE,gen_title,
    "subject",      EXECUTE,gen_subject,
    "operator",     EXECUTE,gen_operator,
    "author",       EXECUTE,gen_author,
    "keywords",     EXECUTE,gen_keywords,
    "doccomm",      EXECUTE,gen_doccomm,
    "version",      EXECUTE,gen_version,
    "nextfile",     EXECUTE,gen_nextfile,
    "ds",           IGNORE, NULL,   /* nicht dokumentierter Befehl !!!  */
    "",             0,      NULL,   /* markiert Tabellenende !!!        */
};

/*------------------------------------------------------------------------
    "Ubersetzungstabelle f"ur Hexadezimal codierte Zeichen im RTF-File
    Bedeutung der Spalten der Tabelle:
        1-3 : Hexcodierung f"ur PC, MAC & ANSI (ASCII) Zeichensaz
        4,5 : Ausgabestring f"ur englische und deutsche LaTeX-Version
    ACHTUNG : Codierung 0 in einer Spalte markiert Tabellenende !!!
------------------------------------------------------------------------*/

struct hexvalue Hextab[]=
{
    0x80,   0,  0,  "\\c{C}",   "\\c{C}",
    0x81,   0,  0,  "\\\"u",    "\"u",
    0x82,   0,  0,  "\\'e",     "\\'e",
    0x83,   0,  0,  "\\^a",     "\\^a",
    0x84,   0,  0,  "\\\"a",    "\"a",
    0x85,   0,  0,  "\\`a",     "\\`a",
    0x86,   0,  0,  "{\\aa}",   "{\\aa}",
    0x87,   0,  0,  "\\c{c}",   "\\c{c}",
    0x88,   0,  0,  "\\^e",     "\\^e",
    0x89,   0,  0,  "\\\"e",    "\"e",
    0x8a,   0,  0,  "\\`e",     "\\`e",
    0x8b,   0,  0,  "\\\"\\i",  "\"\\i",
    0x8c,   0,  0,  "\\^{\\i}", "\\^{\\i}",
    0x8d,   0,  0,  "\\`{\\i}", "\\^{\\i}",
    0x8e,   0,  0,  "\\\"A",    "\"A",
    0x8f,   0,  0,  "{\\AA}",   "{\\AA}",
    0x90,   0,  0,  "\\'E",     "\\'E",
    0x91,   0,  0,  "{\\ae}",   "{\\ae}",
    0x92,   0,  0,  "{\\AE}",   "{\\AE}",
    0x93,   0,  0,  "\\^o",     "\\^o",
    0x94,   0,  0,  "\\\"o",    "\"o",
    0x95,   0,  0,  "\\`o",     "\\`o",
    0x96,   0,  0,  "\\^u",     "\\^u",
    0x97,   0,  0,  "\\`u",     "\\`u",
    0x98,   0,  0,  "\\\"y",    "\"y",
    0x99,   0,  0,  "\\\"O",    "\"O",
    0x9a,   0,  0,  "\\\"U",    "\"U",
    0x9c,   0,  0,  "\\pounds ","\\pounds ",
    0xa0,   0,  0,  "\\'a",     "\\'a",
    0xa1,   0,  0,  "\\'{\\i}", "\\'{\\i}",
    0xa2,   0,  0,  "\\'o",     "\\'o",
    0xa3,   0,  0,  "\\'u",     "\\'u",
    0xa4,   0,  0,  "\\~n",     "\\~n",
    0xa5,   0,  0,  "\\~N",     "\\~N",
    0xa6,   0,  0,  "\\b{a}",   "\\b{a}",
    0xa7,   0,  0,  "\\b{o}",   "\\b{o}",
    0xa8,   0,  0,  "?`",       "?`",
    0xaa,   0,  0,  "$\\neg$",  "$\\neg$",
    0xab,   0,  0,  "$\\frac{1}{2}$","$\\frac{1}{2}$",
    0xac,   0,  0,  "$\\frac{1}{4}$","$\\frac{1}{4}$",
    0xad,   0,  0,  "!`",       "!`",
    0xae,   0,  0,  "$\\ll$",   "$\\ll$",
    0xaf,   0,  0,  "$\\gg$",   "$\\gg$",
    0xe0,   0,  0,  "$\\alpha$","$\\alpha$",
    0xe1,   0,  0,  "{\\ss}",   "\"s",  /* ACHTUNG: <\ss> nicht m"oglich */ 
    0xe2,   0,  0,  "$\\Gamma$","$\\Gamma$",
    0xe3,   0,  0,  "$\\pi$",   "$\\pi$",
    0xe4,   0,  0,  "$\\Sigma$","$\\Sigma$",
    0xe5,   0,  0,  "$\\sigma$","$\\sigma$",
    0xe6,   0,  0,  "$\\mu$",   "$\\mu$",
    0xe7,   0,  0,  "$\\tau$",  "$\\tau$",
    0xe8,   0,  0,  "$\\Phi$",  "$\\Phi$",
    0xe9,   0,  0,  "$\\Theta$","$\\Theta$",
    0xea,   0,  0,  "$\\Omega$","$\\Omega$",
    0xeb,   0,  0,  "$\\delta$","$\\delta$",
    0xec,   0,  0,  "$\\infty$","$\\infty$",
    0xed,   0,  0,  "{\\o}",    "{\\o}",
    0xee,   0,  0,  "$\\in$",   "$\\in$",
    0xef,   0,  0,  "$\\cap$",  "$\\cap$",
    0xf0,   0,  0,  "$\\equiv$","$\\equiv$",
    0xf1,   0,  0,  "$\\pm$",   "$\\pm$",
    0xf2,   0,  0,  "$\\geq$",  "$\\geq$",
    0xf3,   0,  0,  "$\\leq$",  "$\\leq$",
    0xf6,   0,  0,  "$\\div$",  "$\\div$",
    0xf7,   0,  0,  "$\\approx$","$\\approx$",
    0xf8,   0,  0,  "$\\circ$", "$\\circ$",
    0xf9,   0,  0,  "$\\bullet$","$\\bullet$",
    0xfa,   0,  0,  "$\\cdot$", "$\\cdot$",
    0xfb,   0,  0,  "$\\surd$", "$\\surd$",
    0xfc,   0,  0,  "$^{n}$",   "$^{n}$",
    0xfd,   0,  0,  "$^{2}$",   "$^{2}$",
    0x00,   0,  0,  "",         "",     /* markiert Tabellenende !!!    */
};

/*------------------------------------------------------------------------
    "Ubersetzungstabelle f"ur normale Zeichen im RTF-File
    Bedeutung der Spalten der Tabelle:
        1-3 : Hexcodierung f"ur PC, MAC & ANSI (ASCII) Zeichensaz
        4,5 : Ausgabestring f"ur englische und deutsche LaTeX-Version
    ACHTUNG : Codierung 0 in einer Spalte markiert Tabellenende !!!
------------------------------------------------------------------------*/

struct normchar Chartab[]=
{
    '~',    "$\\sim$",
    '\\',   "$\\backslash$",
    '|',    "$|$",
    '$',    "\\$",
    '%',    "\\%%",
    '{',    "\\{",
    '}',    "\\}",
    '_',    "\\_",
    '#',    "\\#",
    '&',    "\\&",
    '"',    "''",
    0,      "",                         /* markiert Tabellenende !!!    */
};

/*------------------------------ E N D E -------------------------------*/
