/*
 * $Id$
 */

/*------------------------------------------------------------------------

    RTF2TEX: RTF-Dateien aus WORD5.0 in TeX/LaTeX konvertieren

    Hauptprogramm, Token-Scanner und Zeilenumbruchsteuerung
    
    letzte "Anderung: am 4.9.1991 um 17:24

    History:

    V0.10   MH 21.8.1991
            Grundger"ust programmiert
    V0.20   MH 22.8.1991
            Scanner implementiert
    V0.30   MH 23.8.1991
            '-v' Verbose Option implementiert
    V0.40   MH 24.8.1991
            Codegenerierung: Grundgr"ust und Textattribute
    V0.50   MH 26.8.1991
            '-i', '-s', '-p' Optionen implementiert
    V0.60   MH 27.8.1991
            "Ubersetzungstabelle f"ur Hexadezimalwerte erstellt
            Diverse Optimierungen
    V0.70   MH 2.9.1991
            Kommentierung "uberarbeitet f"ur endg"ultige Version
    V1.00   MH 3.9.1991
            Endg"ultige Version f"ur Studienarbeit
                                                                                                                                        
------------------------------------------------------------------------*/

/*------------------------------------------------------------------------
    Includes ....
------------------------------------------------------------------------*/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "rtf2tex.h"
#include "getopt.h"

#ifdef AMIGA
#include "rtf2tex_rev.h"
const version_string = VERSTAG;
#endif

/*------------------------------------------------------------------------
    globale Variablen ....
------------------------------------------------------------------------*/

extern struct command Cmdtab[];     /* Tabelle mit RTF-Kommandos        */

extern struct hexvalue Hextab[];    /* Tabelle f"ur Hexzahlen"ubersetzung */

extern struct normchar Chartab[];   /* "Ubersetzungstabelle f"ur Zeichen  */

/*----------------------------------------------------------------------*/

int Verbose = FALSE,                /* Option 'verbose message output'  */
    German = FALSE,                 /* LaTeX Option 'german'            */
    Include = FALSE,                /* Include-Datei erzeugen           */
    Switchattribut = FALSE,         /* Textattributumschaltung an       */
    Hochtiefflag = FALSE,           /* Hoch- oder Tiefstellung an       */
    Footnoteflag = FALSE,           /* Fu"snote an                       */
    Newleftskip = FALSE,            /* \leftskip ge"andert               */
    Newrightskip = FALSE,           /* \rightskip ge"andert              */
    Newparindent = FALSE,           /* \parindent ge"andert              */
    Noblocksatz = FALSE,            /* kein Blocksatz im Absatz         */
    Smallcaps = FALSE,              /* Merker f"ur Textattribute         */
    Boldface = FALSE,
    Italic = FALSE,
    Parflag = FALSE,                /* Wird mit neuem Absatz gesetzt    */
    Formatset = FALSE,              /* Formatierung f"ur Absatz gsetzt
                                       Wurde diese nicht am Absatzanfang
                                       gesetzt, dann Linksb"undig !!!    */
    Textcode = DEFAULTCODE,         /* Defaulteinstellung Textcodierung
                                       Ben"otigt f"ur Tabellenzugriff auf
                                       Hexadezimal codierte Zeichen     */
    Linepos = 0,                    /* Position innerhald einer Zeile   */
    Pardepth = 0,                   /* Verschachtelungstiefe Absatz-{
                                       Dieser Z"ahler wird "uber \pard
                                       und \par gesteuert               */
    Underlinedepth = 0;             /* Verschachtelung \underline       */

FILE *Ausgabefile = NULL,           /* Descriptor f"ur Ausgabefile       */
     *Eingabefile = NULL;           /* Descriptor f"ur Eingabefile       */
         
struct scantoken Token;             /* Struktur mit gescanntem Token    */

/*-----------------------------------------------------------------------
    Systemmeldungen ....
------------------------------------------------------------------------*/

    /* ACHTUNG: niemals eine Zeile einf"ugen !!! */

const    
char *sysmsg[]=
     {
        /* 00 */ "Input filename missing!\n",
        /* 01 */ "Too many parameters in commandline!\n",
        /* 02 */ "Input file '%s' not found!\n",
        /* 03 */ "rtf2tex converter aborted.\n",
        /* 04 */ "rtf2tex converter successfully terminated.\n",
        /* 05 */ "Converting '%s' -> '%s' ....\n",
        /* 06 */ "Cannot open output file '%s'!\n",
        /* 07 */ "Unknown hexvalue: <\\'%s>!\n",
        /* 08 */ "Unknown command: <\\%s>!\n",
        /* 09 */ "Command <\\%s> not yet implemented!\n",
        /* 10 */ "Document style missing!\n"
     };

    /* Hier steht der Text f"ur die Kurzanleitung, letzte Zeile ist leer */

const 
char *usagetxt[]=
     {
       "usage: rft2tex [-ghiv] [-o outfile] [-p style-option] [-s doc-style] infile\n",
        ""
     };

const 
char *verboseusage[]=
    {
       "RFT2TEX  ---  RTF -> LaTeX Converter\n\n",
       "usage:   rft2tex [-ghiv] [-o outfile] [-p style-option] [-s doc-style] infile\n",
       "                  -g           generate output for German style\n",
       "                  -h           help --- show this text\n",
       "                  -i           don't generate \\documentstyle header\n",
       "                  -v           verbose\n",
       "                  -o outfile   write output to file (default: stdout)\n",
       "                  -p option    use as document style option\n",
       "                  -s style     use as document style\n",
       ""
    };
    
/*-----------------------------------------------------------------------
    Hauptprogramm
------------------------------------------------------------------------*/

int main(argc,argv)
    int argc;
    char **argv;
{
    int zeichen,                    /* Zeichen einer Option             */
        tabpos,                     /* Position in Tabelle              */
        length;                     /* Zeichen eines Ausgabestrings     */
                
    char *ausgabefilename = "",     /* Zeiger auf den Ausgabedateinamen */
         *eingabefilename = "",     /* Zeiger auf den Eingabedateinamen */
         *documentstyle = DEFAULTSTYLE,         /* Zeiger auf Style     */
         *documentoptions = DEFAULTDOCOPT;      /* Zeiger auf Optionen  */
             
    opterr = 0;             /* keine Meldung bei falschen Optionen aus
                               der Funktion `getopt()' ausgeben. Diese
                               Variable ist in der Datei `getopt.c' als
                               `static int' deklariert */
    
    /* Kommandozeile nach Parametern durchsuchen */

    while((zeichen = getopt(argc,argv,"ghivo:p:s:")) != -1)
    {
        switch(zeichen)
        {
            case 'h':           
               Verbose = TRUE;
               printusage();
               /**NOT REACHED**/
               break;

            case '?':           /* stammt von getopt() */
                printusage();
                break;
                
            case 'o':
                ausgabefilename = optarg;
                break;
            
            case 'p':
                documentoptions = optarg;
                break;
            
            case 's':
                documentstyle = optarg;
                break;
            
            case 'g':
                German = TRUE;
                break;              

            case 'v':
                Verbose = TRUE;
                break;

            case 'i':
                Include = TRUE;
                break;
        }
    }

    if(argc<=1)
        printusage();   /* Hilfestellung und exit */

    if(optind<argc)
        eingabefilename = argv[optind++];
    else
        errout(sysmsg[0],"",TRUE);      /* Eingabefilename fehlt */
    if(optind<argc)
        errout(sysmsg[1],"",TRUE);      /* Zu viele Parameter */

    if((Eingabefile = fopen(eingabefilename,"r")) == NULL)
        errout(sysmsg[2],eingabefilename,TRUE);     /* not found */

    if(*ausgabefilename != '\0')
    {
        if((Ausgabefile = fopen(ausgabefilename,"w")) == NULL)
            errout(sysmsg[6],ausgabefilename,TRUE);     /* not open */
    }
    else
    {
        Ausgabefile = stdout;       /* kein '-o ...' -> Bildshirm */
        ausgabefilename = "no name";
    }
            
    if(Verbose)     
        fprintf(stdout,sysmsg[5],eingabefilename,ausgabefilename);

    /* Infozeile als Kommentar */
    
    fprintf(Ausgabefile,
        "%% Datei: '%s', erzeugt aus Datei '%s' mit RTF2TEX\n\n",
        ausgabefilename,eingabefilename);
    
    /* Init-Zeile f"ur TeX-Dokumente */

    if(!Include)
    {       
        fprintf(Ausgabefile,"\\documentstyle[");
        if(German && !*documentoptions)
            fprintf(Ausgabefile,"german");      /* german.sty */
        if(*documentoptions)
            fprintf(Ausgabefile,"%s",documentoptions);
        if(!*documentstyle)
            errout(sysmsg[10],"",TRUE);         /* Document style fehlt */
        fprintf(Ausgabefile,"]{%s}\n\n",documentstyle);

        /* Dokument Anfang */
        fprintf(Ausgabefile,
            "\\parindent 0pt \\leftskip 0pt \\rightskip 0pt\n\n");
        fprintf(Ausgabefile,"\\begin{document}\n\n");
    }
                
    while(gettoken())
    {
        switch(Token.type)
        {
            case NORMCMD:
            case PARMCMD:
                tabpos = 0;
                while(*Cmdtab[tabpos].cmd)
                {
                    if(!strcmp(Cmdtab[tabpos].cmd,Token.value.cmd.text))
                    {
                        switch(Cmdtab[tabpos].state)
                        {
                            case COMMENT:
                                fprintf(Ausgabefile," %% \\%s",
                                    Token.value.cmd.text);
                                checknewline(LINEBREAK,!BREAKALLOWED);
                                    /* Umbruch nach Kommentar */
                                break;
                            
                            case EXECUTE:
                                Cmdtab[tabpos].func();
                                break;
                            
                            case IGNORE:
                                break;
                            
                            case WARN:
                                errout(sysmsg[9],
                                    Token.value.cmd.text,FALSE);
                                break;
                        }
                        break;
                    }
                    else
                        tabpos++;
                }
                if(!*Cmdtab[tabpos].cmd)    /* Unbekanntes Kommando */
                {
                    errout(sysmsg[8],Token.value.cmd.text,FALSE);
                    fprintf(Ausgabefile," %% \\%s",Token.value.cmd.text);
                    checknewline(LINEBREAK,!BREAKALLOWED);
                        /* Umbruch nach Kommentar */
                }
                break;

            case NORMCHAR:
                if(!Formatset)
                {
                    Formatset = TRUE;
                    if(!Footnoteflag)
                        Noblocksatz = TRUE;
                    length=fprintf(Ausgabefile,"\\raggedright ");
                    checknewline(length,BREAKALLOWED);
                }
                tabpos = 0;
                while(Chartab[tabpos].value)
                {
                    if(Chartab[tabpos].value == Token.value.zeichen)
                    {
                        length=fprintf(Ausgabefile,
                            Chartab[tabpos].outstring);
                        checknewline(length,*(Chartab[tabpos].outstring+
                            (int)strlen(Chartab[tabpos].outstring)-1));
                        break;                      
                    }
                    else
                        tabpos++;
                }
                if(!Chartab[tabpos].value)
                {
                    length=fprintf(Ausgabefile,"%c",Token.value.zeichen);
                    checknewline(length,Token.value.zeichen);
                }
                Parflag = FALSE;
                Parflag = FALSE;
                break;
            
            case HEXVALUE:
                tabpos = 0;
                while(Hextab[tabpos].value[Textcode])
                {
                    if(Hextab[tabpos].value[Textcode] == 
                        Token.value.zeichen)
                    {
                        length=fprintf(Ausgabefile,
                            Hextab[tabpos].outstring[German]);
                        checknewline(length,
                            *(Hextab[tabpos].outstring[German]+(int)
                            strlen(Hextab[tabpos].outstring[German])-1));
                        break;
                    }
                    else
                        tabpos++;
                }
                if(!Hextab[tabpos].value[Textcode]) /* Hexwert Unbekannt */
                {
                    errout(sysmsg[7],Token.value.cmd.text,FALSE);
                    fprintf(Ausgabefile," %% \\'%s",Token.value.cmd.text);
                    checknewline(LINEBREAK,!BREAKALLOWED);  
                        /* Umbruch nach Kommentar */
                }
                Parflag = FALSE;
                break;

            case GROUPBEGIN:
                Parflag = FALSE;
                break;
            
            case GROUPEND:
                if(Footnoteflag)
                {
                    Footnoteflag = FALSE;
                    length=fprintf(Ausgabefile,"}");
                    checknewline(length,BREAKALLOWED);
                }
                Parflag = FALSE;
                break;
        }
    }
    
    /* Dokument Ende */

    if(!Parflag)
        gen_par();          /* Umgebungen schlie"sen */

    if(!Include)    
        fprintf(Ausgabefile,"\n\\end{document}\n");
            
    if(Eingabefile != NULL)
        fclose(Eingabefile);
    if(Ausgabefile != NULL && Ausgabefile != stdout)
        fclose(Ausgabefile);

    if(Verbose)     
        fprintf(stdout,sysmsg[4]);      /* erfolgreich beendet */
    return(0);
}

/*-----------------------------------------------------------------------
    Hilfestellung ausgeben (bei falschen oder fehlenden Optionen)
------------------------------------------------------------------------*/

void printusage()
{
    int line=0;
    
    if(Verbose)
        while(*verboseusage[line] != '\0')
            fprintf(stdout,verboseusage[line++]);
    else
        while(*usagetxt[line] != '\0')
            fprintf(stdout,usagetxt[line++]);
    exit(USAGEEXIT);
}

/*-----------------------------------------------------------------------
    Fehlermeldung auf `stdout' ausgeben
------------------------------------------------------------------------*/

void errout(errstring,optparm,exitflag)
    char *errstring,                    /* Fehlermeldung */
         *optparm;                      /* optionaler Parameter */
    int exitflag;                       /* Programmabbruch wenn TRUE */
{
    fprintf(stderr,errstring,optparm);
    if(exitflag==TRUE)                  /* Programmabbruch */
    {
        if(Verbose)
            fprintf(stdout,sysmsg[3]);
        if(Eingabefile != NULL)
            fclose(Eingabefile);
        if(Ausgabefile != NULL)
            fclose(Ausgabefile);
        exit(ERROREXIT);
    }
    return;
}

/*-----------------------------------------------------------------------
    Token aus Eingabedatei scannen
    liefert FALSE, wenn Dateiende erreicht, sonst TRUE
------------------------------------------------------------------------*/

int gettoken()
{
    char zeichen;
    int  position,              /* Zeichenposition im Scannerdurchlauf  */
         scanmode = SCANBEGIN;
        
    while((zeichen = getc(Eingabefile)) != EOF)
    {
        if(zeichen == '\n')     /* CR/LF ignorieren */
            continue;
            
        switch(scanmode)
        {
            case SCANBEGIN:
                switch(zeichen)
                {
                    case '\\':
                        position = 0;
                        Token.type = NORMCMD;
                        scanmode = SCANCMD;
                        break;
                    
                    case '{':
                        Token.type = GROUPBEGIN;
                        Token.value.zeichen = zeichen;
                        return(TRUE);
                    
                    case '}':
                        Token.type = GROUPEND;
                        Token.value.zeichen = zeichen;
                        return(TRUE);
                    
                    default:
                        Token.type = NORMCHAR;
                        Token.value.zeichen = zeichen;
                        return(TRUE);
                }
                break;
            
            case SCANCMD:
                switch(zeichen)
                {
                    case '\\':          /* Sonderzeichen */
                    case '{':
                    case '}':
                        if(!position)
                        {
                            Token.type = NORMCHAR;
                            Token.value.zeichen = zeichen;
                            return(TRUE);
                        }
                        Token.value.cmd.text[position] = '\0';
                        ungetc(zeichen,Eingabefile);
                        return(TRUE);
                                
                    case '\'':          /* 2-stellige Hexzahl */
                        if(!position)
                        {
                            Token.type = HEXVALUE;
                            scanmode = SCANHEX;
                            position = 0;
                            break;
                        }
                        Token.value.cmd.text[position] = '\0';
                        ungetc(zeichen,Eingabefile);
                        return(TRUE);

                    case ' ':           /* Delimiter f"ur \command */
                        Token.value.cmd.text[position] = '\0';
                        return(TRUE);

                    case '-':           /* Delimiter, neg. Parameter */
                        scanmode = SCANPARM;
                        Token.type = PARMCMD;
                        Token.value.cmd.text[position] = '\0';
                        position = 0;
                        Token.value.cmd.parm[position++] = zeichen;
                        break;
                                                
                    default:            /* Zeichen zu \command */
                        if(isalpha(zeichen))
                        {
                            Token.value.cmd.text[position++] = zeichen;
                            break;
                        }
                        if(isdigit(zeichen))
                        {
                            scanmode = SCANPARM;
                            Token.type = PARMCMD;
                            Token.value.cmd.text[position] = '\0';
                            position = 0;
                            Token.value.cmd.parm[position++] = zeichen;
                            break;
                        }
                        Token.value.cmd.text[position] = '\0';
                        ungetc(zeichen,Eingabefile);
                        return(TRUE);
                }
                break;
                
            case SCANPARM:
                switch(zeichen)
                {
                    case ' ':
                        Token.value.cmd.parm[position] = '\0';
                        return(TRUE);

                    default:
                        if(isdigit(zeichen))
                        {
                            Token.value.cmd.parm[position++] = zeichen;
                            break;
                        }
                        Token.value.cmd.parm[position] = '\0';
                        ungetc(zeichen,Eingabefile);
                        return(TRUE);
                }
                break;

            case SCANHEX:
                if(position < 2)
                    Token.value.cmd.text[position++] = zeichen;
                else
                {
                    Token.value.cmd.text[position] = '\0';
                    ungetc(zeichen, Eingabefile);
                    Token.value.zeichen = 
                        (int)hexval(Token.value.cmd.text);
                    return(TRUE);
                }
                break;              
        }
    }
    return(FALSE);      /* Dateiende erreicht */
}

/*-----------------------------------------------------------------------
    Hexadezimalstring in Zahl umwandeln
------------------------------------------------------------------------*/

long hexval(string)
    register char *string;
{
    register int pos;
    register char *zeiger;
    register long wert=0,
                  fakt=1;

    for(pos=(int)strlen(string)-1;pos>=0;pos--)
    {
        zeiger=string+pos;
        if( *zeiger >= '0' && *zeiger <= '9' )
            wert += fakt * (*zeiger - '0');
        if( *zeiger >= 'a' && *zeiger <= 'f' )
            wert += fakt * (*zeiger - 'a' + 10);
        if( *zeiger >= 'A' && *zeiger <= 'F' )
            wert += fakt * (*zeiger - 'A' + 10);
        fakt *= 16;
    }
    return(wert);
}

/*-----------------------------------------------------------------------
    Zeilenumbruch steuern
    wenn lastsize==LINEBREAK    -> direkter Umbruch
    sonst Umbruch, wenn Zeilenl"ange "uberschritten
------------------------------------------------------------------------*/

void checknewline(lastsize, lastchar)
    int lastsize,                   /* L"ange der letzten Ausgabe        */
        lastchar;                   /* letztes ausgegebenes Zeichen     */
{
    Linepos += lastsize;
    if( (Linepos >= LINELENGTH && lastchar == BREAKALLOWED) ||
        (lastsize == LINEBREAK) )
    {
        fprintf(Ausgabefile,"\n");
        Linepos = 0;
    }
    return;
}
        
/*------------------------------ E N D E -------------------------------*/
